import time
import board
import displayio
import terminalio
import gifio
import os, gc
from adafruit_matrixportal.matrixportal import MatrixPortal
from adafruit_apds9960.apds9960 import APDS9960
from digitalio import DigitalInOut, Pull
from adafruit_debouncer import Debouncer

# Directories for files
GIF_DIRECTORY = "/gifs/"
IMAGE_DIRECTORY = "/images/"

# Initialize MatrixPortal and hardware components
matrixportal = MatrixPortal(status_neopixel=board.NEOPIXEL, bit_depth=6)
display = matrixportal.display
splash = matrixportal.splash
content_group = displayio.Group()
splash.append(content_group)

# Button setup
button_a_pin = DigitalInOut(board.BUTTON_UP)
button_a_pin.switch_to_input(Pull.UP)
button_a = Debouncer(button_a_pin)

button_b_pin = DigitalInOut(board.BUTTON_DOWN)
button_b_pin.switch_to_input(Pull.UP)
button_b = Debouncer(button_b_pin)

# Gesture sensor setup
i2c = board.STEMMA_I2C()
apds = APDS9960(i2c)
apds.enable_proximity = True
apds.enable_gesture = True

# Modes and options
MODES = ['GIF', 'Image', 'Image with Text', 'Solid Color with Text']
current_mode_index = 0  # Start with GIF Mode

# File lists for images and GIFs
def get_files(base):
    allfiles = os.listdir(base)
    file_names = []
    for _, filetext in enumerate(allfiles):
        if not filetext.startswith("."):
            if filetext not in ('boot_out.txt', 'System Volume Information'):
                if filetext.endswith(".gif") or filetext.endswith(".bmp"):
                    file_names.append(filetext)
    return file_names

gifs = get_files(GIF_DIRECTORY)
images = get_files(IMAGE_DIRECTORY)

# Text overlay options and settings
texts = ["GO TEAM!", "ROCK THE RED", "SEIVE", "REF YOU SUCK!!"]
text_index = 0
text_mode = 'static'
text_visible = True
text_color_index = 0

# Can add as many as you want
text_colors = [0xFF0000, 0x00FF00, 0x0000FF, 0xFFFFFF, 0xFFFF00]  # Red, Green, Blue, White, Yellow
background_colors = [0x000000, 0xFF00FF, 0x00FFFF, 0xFF8000, 0x008000]  # Black, Magenta, Cyan, Orange, Green
background_color_index = 0

gif_index = 0
image_index = 0

needs_update = True
def switch_mode(direction):
    global current_mode_index, needs_update
    if direction == 'next':
        current_mode_index = (current_mode_index + 1) % len(MODES)
    elif direction == 'previous':
        current_mode_index = (current_mode_index - 1) % len(MODES)
    print("Switched to", MODES[current_mode_index], "Mode")
    needs_update = True

def next_gif():
    global gif_index, needs_update
    gif_index = (gif_index + 1) % len(gifs)
    needs_update = True

def previous_gif():
    global gif_index, needs_update
    gif_index = (gif_index - 1) % len(gifs)
    needs_update = True

def next_image():
    global image_index, needs_update
    image_index = (image_index + 1) % len(images)
    needs_update = True

def previous_image():
    global image_index, needs_update
    image_index = (image_index - 1) % len(images)
    needs_update = True

def next_text():
    global text_index, needs_update
    text_index = (text_index + 1) % len(texts)
    needs_update = True

def previous_text():
    global text_index, needs_update
    text_index = (text_index - 1) % len(texts)
    needs_update = True

def next_background_color():
    global background_color_index, needs_update
    background_color_index = (background_color_index + 1) % len(background_colors)
    needs_update = True

def previous_background_color():
    global background_color_index, needs_update
    background_color_index = (background_color_index - 1) % len(background_colors)
    needs_update = True

def next_text_color():
    global text_color_index, needs_update
    text_color_index = (text_color_index + 1) % len(text_colors)
    needs_update = True

def toggle_text_mode():
    global text_mode, needs_update
    text_mode = 'scroll' if text_mode == 'static' else 'static'
    needs_update = True

def clear_display():
    matrixportal.remove_all_text()
    while content_group:
        content_group.pop()

def update_display():
    global needs_update
    clear_display()
    needs_update = False
    if MODES[current_mode_index] == 'GIF':
        if gifs:
            gif_path = GIF_DIRECTORY + gifs[gif_index]
            play_gif(gif_path)
        else:
            print("No GIFs found.")
    elif MODES[current_mode_index] == 'Image':
        if images:
            image_path = IMAGE_DIRECTORY + images[image_index]
            matrixportal.set_background(image_path)
            while not needs_update:
                if check_inputs():
                    pass
                time.sleep(0.05)
        else:
            print("No images found.")
    elif MODES[current_mode_index] == 'Image with Text':
        if images:
            image_path = IMAGE_DIRECTORY + images[image_index]
            matrixportal.set_background(image_path)
            display_text(texts[text_index], text_colors[text_color_index])
        else:
            print("No images found.")
    elif MODES[current_mode_index] == 'Solid Color with Text':
        color = background_colors[background_color_index]
        matrixportal.set_background(color)
        display_text(texts[text_index], text_colors[text_color_index])

def play_gif(file_path):
    gc.collect()
    try:
        odg = gifio.OnDiskGif(file_path)
    except Exception as e:
        print("Failed to load GIF:", e)
        return
    gif_tile = displayio.TileGrid(
        odg.bitmap,
        pixel_shader=displayio.ColorConverter(
            input_colorspace=displayio.Colorspace.RGB565_SWAPPED
        ),
    )
    content_group.append(gif_tile)
    try:
        while not needs_update:
            start_time = time.monotonic()
            delay = odg.next_frame()
            if delay == 0:
                break  # End of GIF
            elapsed = 0
            # Non-blocking delay loop
            while elapsed < delay and not needs_update:
                if check_inputs():
                    pass  # Exit function immediately to change GIF
                time.sleep(0.01)  # Small sleep to prevent CPU overuse
                elapsed = time.monotonic() - start_time
    finally:
        if gif_tile in content_group:
            content_group.remove(gif_tile)
        odg.deinit()
        gc.collect()

def display_text(message, color):
    matrixportal.remove_all_text()
    if text_mode == 'scroll':
        matrixportal.add_text(
            text_font=terminalio.FONT,
            text_position=(display.width, display.height // 2 - 1),
            scrolling=True,
            text_color=color,
        )
        matrixportal.set_text(message)
        while not needs_update:
            matrixportal.scroll()
            if check_inputs():
                pass
            time.sleep(0.02)
        matrixportal.remove_all_text()
    else:
        text_area = matrixportal.add_text(
            text_font=terminalio.FONT,
            text_position=(display.width // 2, display.height // 2 - 1),
            text_anchor_point=(0.5, 0.5),
            text_color=color,
        )
        matrixportal.set_text(message)
        while not needs_update:
            if check_inputs():
                pass
            time.sleep(0.05)
        matrixportal.remove_all_text()

def check_inputs():
    global needs_update
    button_a.update()
    button_b.update()
    gesture = apds.gesture()
    input_detected = False

    if gesture == 0x03:  # Left Gesture
        switch_mode('previous')
        input_detected = True
    elif gesture == 0x04:  # Right Gesture
        switch_mode('next')
        input_detected = True
    elif gesture == 0x01:  # Up Gesture
        toggle_text_mode()
        input_detected = True
    elif gesture == 0x02:  # Down Gesture
        next_text_color()
        input_detected = True

    if MODES[current_mode_index] == 'GIF':
        if button_a.fell:
            next_gif()
            input_detected = True
        elif button_b.fell:
            previous_gif()
            input_detected = True
    elif MODES[current_mode_index] == 'Image':
        if button_a.fell:
            next_image()
            input_detected = True
        elif button_b.fell:
            previous_image()
            input_detected = True
    elif MODES[current_mode_index] == 'Image with Text':
        if button_a.fell:
            next_image()
            input_detected = True
        elif button_b.fell:
            next_text()
            input_detected = True
    elif MODES[current_mode_index] == 'Solid Color with Text':
        if button_a.fell:
            next_background_color()
            input_detected = True
        elif button_b.fell:
            next_text()
            input_detected = True

        if input_detected:
            needs_update = True

    return input_detected

while True:
    if needs_update:
        update_display()
    else:
        if check_inputs():
            pass  # Actions are handled in check_inputs
    time.sleep(0.05)